package de.clab.osami.conferencing.vlc.ServiceInterfaces;

import java.awt.Dimension;
import java.awt.Panel;

/**
 * The interface IVideoConference
 * @author Christian Thanos
 *
 */
public interface IVideoConference {
	/**
	 * Start the VC.
	 *
	 * @param ipv4 the IPv4 address the VC will connect to
	 * @return null on success or a string containing an error message on failure
	 */
	public String start(String ipv4);

	/**
	 * Start the VC.
	 *
	 * @return null on success or a string containing an error message on failure
	 */
	public String start();

	/**
	 * Stop the VC.
	 *
	 * @return null on success or a string containing an error message on failure
	 */
	public String stop();
	
	/**
	 * Gets the IP of the VC.
	 *
	 * @return the IP of the VC
	 */
	public String getIp();
	
	/**
	 * Sets the IP of the VC.
	 *
	 * @param ipv4 the ipv4
	 * @return the string
	 */
	public String setIp(String ipv4);
	
	/**
	 * Gets the target port.
	 *
	 * @return null on success or a string containing an error message on failure
	 */
	public int getTargetPort();
	
	/**
	 * Sets the target port.
	 *
	 * @param port the port
	 * @return null on success or a string containing an error message on failure
	 */
	public String setTargetPort(int port);
	
	/**
	 * Gets the local port.
	 *
	 * @return the local port
	 */
	public int getLocalPort();
	
	/**
	 * Sets the local port.
	 *
	 * @param port the port
	 * @return null on success or a string containing an error message on failure
	 */
	public String setLocalPort(int port);
	
	// Caching
	/**
	 * Gets the capture caching.
	 *
	 * @return the capture caching
	 */
	public int getCaptureCaching();
	
	/**
	 * Sets the capture caching.
	 *
	 * @param ms the time in milliseconds
	 * @return null on success or a string containing an error message on failure
	 */
	public String setCaptureCaching(int ms);
	
	/**
	 * Gets the receive caching.
	 *
	 * @return the receive caching
	 */
	public int getReceiveCaching();
	
	/**
	 * Sets the receive caching.
	 *
	 * @param ms the time in milliseconds
	 * @return null on success or a string containing an error message on failure
	 */
	public String setReceiveCaching(int ms);
	
	//inputDevice
	
	/**
	 * Gets the video input devices.
	 *
	 * @return the video input devices
	 */
	public String[] getVideoInputDevices();
	
	/**
	 * Gets the current video input device.
	 *
	 * @return the current video input device
	 */
	public String getCurrentVideoInputDevice();
	
	/**
	 * Sets the video input device.
	 *
	 * @param device the device
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoInputDevice(String device);
	
	/**
	 * Gets the audio input devices.
	 *
	 * @return the audio input devices
	 */
	public String[] getAudioInputDevices();
	
	/**
	 * Gets the current audio input device.
	 *
	 * @return the current audio input device
	 */
	public String getCurrentAudioInputDevice();
	
	/**
	 * Sets the audio input device.
	 *
	 * @param device the device
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioInputDevice(String device);

	// Audio and Video enabling
	/**
	 * Enables or disables the video enabled.
	 *
	 * @param flag the flag
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoEnabled(boolean flag);
	
	/**
	 * Checks if video is enabled.
	 *
	 * @return true, if video is enabled
	 */
	public boolean isVideoEnabled();
	
	/**
	 * Enables or disables the audio.
	 *
	 * @param flag the flag
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioEnabled(boolean flag);
	
	/**
	 * Checks if audio is enabled.
	 *
	 * @return true, if audio is enabled
	 */
	public boolean isAudioEnabled();

	// Codec configuration
	/**
	 * Sets the video codec.
	 *
	 * @param codec the codec
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoCodec(String codec);
	
	/**
	 * Gets the current video codec.
	 *
	 * @return the current video codec
	 */
	public String getCurrentVideoCodec();
	
	/**
	 * Gets the supported video codecs.
	 *
	 * @return the supported video codecs
	 */
	public String[] getSupportedVideoCodecs();
	
	/**
	 * Sets the audio codec.
	 *
	 * @param codec the codec
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioCodec(String codec);
	
	/**
	 * Gets the current audio codec.
	 *
	 * @return the current audio codec
	 */
	public String getCurrentAudioCodec();
	
	/**
	 * Gets the supported audio codecs.
	 *
	 * @return the supported audio codecs
	 */
	public String[] getSupportedAudioCodecs();
	
	// Bitrate configuration
	
	/**
	 * Sets the video bit rate.
	 *
	 * @param bitrate the video bitrate
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoBitRate(int bitrate);
	
	/**
	 * Gets the video bitrate.
	 *
	 * @return the video bitrate
	 */
	public int getVideoBitrate();
	
	/**
	 * Sets the audio bit rate.
	 *
	 * @param bitrate the audio bitrate
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioBitRate(int bitrate);
	
	/**
	 * Gets the audio bitrate.
	 *
	 * @return the audio bitrate
	 */
	public int getAudioBitrate();

	// Video specific configuration
	/**
	 * Sets the video frame rate.
	 *
	 * @param framerate the video framerate
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoFrameRate(int framerate);
	
	/**
	 * Gets the video frame rate.
	 *
	 * @return the video frame rate
	 */
	public int getVideoFrameRate();
	
	/**
	 * Sets the video size.
	 *
	 * @param size the video size
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoSize(Dimension size);
	
	/**
	 * Gets the video size.
	 *
	 * @return the video size
	 */
	public Dimension getVideoSize();
	
	/**
	 * Gets the video scale.
	 *
	 * @return the video scale
	 */
	public float getVideoScale();
	
	/**
	 * Sets the video scale.
	 *
	 * @param scale the video scale
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoScale(float scale);
	
	// Video output configuration
	/**
	 * Gets the video local component.
	 *
	 * @return the video local component
	 */
	public Object getVideoLocalComponent();
	
	/**
	 * Sets the video local component.
	 *
	 * @param surface the surface
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoLocalComponent(Object surface);
	
	/**
	 * Gets the video remote component.
	 *
	 * @return the video remote component
	 */
	public Object getVideoRemoteComponent();
	
	/**
	 * Sets the video remote component.
	 *
	 * @param surface the surface
	 * @return null on success or a string containing an error message on failure
	 */
	public String setVideoRemoteComponent(Object surface);
	
	
	// Audio specific configuration
	
	/**
	 * Gets the audio channels.
	 *
	 * @return the audio channels
	 */
	public int getAudioChannels();
	
	/**
	 * Sets the audio channels.
	 *
	 * @param channels the channels
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioChannels(byte channels);
	
	/**
	 * Gets the audio sample rate.
	 *
	 * @return the audio sample rate
	 */
	public int getAudioSampleRate();
	
	/**
	 * Sets the audio sample rate.
	 *
	 * @param sampleRate the sample rate
	 * @return null on success or a string containing an error message on failure
	 */
	public String setAudioSampleRate(int sampleRate);
		
	//logging
	/**
	 * Enables or disables the logging.
	 *
	 * @param flag the flag that determines whether logging should be enabled or disabled
	 * @return null on success or a string containing an error message on failure
	 */
	public String setLoggingEnabled(boolean flag);
	
	// Configuration Panel
	/**
	 * Gets the configuration panel.
	 *
	 * @return the configuration panel
	 */
	public Panel getConfigurationPanel();
	
	/**
	 * Load configuration.
	 *
	 * @param location the location
	 * @return null on success or a string containing an error message on failure
	 */
	public String loadConfiguration(String location);
	
	/**
	 * Save configuration.
	 *
	 * @param location the location
	 * @return null on success or a string containing an error message on failure
	 */
	public String saveConfiguration(String location);
	
	/**
	 * Reconfigure the VC.
	 *
	 * @return null on success or a string containing an error message on failure
	 */
	public String reconfigure();

	/**
	 * @return
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#getLocalIp()
	 */
	public String getStreamingLocalIp();

	/**
	 * @param localIp
	 * @see de.clab.osami.conferencing.vlc.config.IConfiguration#setLocalIp(java.lang.String)
	 */
	public void setStreamingLocalIp(String localIp);
	
	
	/**
	 * Enables or disables the local video component
	 * @param flag
	 */
	public void setVideoLocalEnabled(Boolean flag);
	
	/**
	 * Enables or disables the remote video component
	 * @param flag
	 */
	public void setVideoRemoteEnabled(Boolean flag);

}
